@@ -29,36 +29,22 @@ sub new {
     my $m = Graph::BitMatrix->new($g);
     my $self = bless [ $m, undef, \@V ], $class;
     return $self if !$want_distance;
-    # for my $u (@V) {
-    #     for my $v (@V) {
-    #         if ($g->has_edge($u, $v)) {
-    #             $n->set($u, $v,
-    #                    $g->get_edge_attribute($u, $v, $d));
-    #        }
-    #     }
-    # }
     my $n = $self->[ _DM ] = Graph::Matrix->new($g);
     $n->set($_, $_, 0) for @V;
-    my $Vi = $g->[_V][_i];
-    my $Ei = $g->[_E][_i];
-    my %V; @V{ @V } = 0 .. $#V;
     my $n0 = $n->[0];
     my $n1 = $n->[1];
     my $undirected = $g->is_undirected;
     my $multiedged = $g->multiedged;
-    for (my $e = $#$Ei; $e >= 0; $e--) {
-	next if !defined $Ei->[ $e ];
-	my ($i0, $j0) = @{ $Ei->[ $e ] };
-	my $i1 = $V{ $Vi->[ $i0 ] };
-	my $j1 = $V{ $Vi->[ $j0 ] };
-	my $u = $V[ $i1 ];
-	my $v = $V[ $j1 ];
-	$n0->[ $i1 ]->[ $j1 ] = $multiedged
+    for my $e ($g->edges) {
+	my ($u, $v) = @$e;
+        $n->set($u, $v, $multiedged
 	    ? _multiedged_distances($g, $u, $v, $d)
-	    : $g->get_edge_attribute($u, $v, $d);
-	$n0->[ $j1 ]->[ $i1 ] = $multiedged
+	    : $g->get_edge_attribute($u, $v, $d)
+        );
+        $n->set($v, $u, $multiedged
 	    ? _multiedged_distances($g, $v, $u, $d)
-	    : $g->get_edge_attribute($v, $u, $d) if $undirected;
+	    : $g->get_edge_attribute($v, $u, $d)
+        ) if $undirected;
     }
     $self;
 }
