@@ -1,7 +1,7 @@
 package Graph::Matrix;
 
-# $SIG{__DIE__ } = sub { use Carp; confess };
-# $SIG{__WARN__} = sub { use Carp; confess };
+# $SIG{__DIE__ } = \&Graph::__carp_confess;
+# $SIG{__WARN__} = \&Graph::__carp_confess;
 
 use strict;
 use warnings;
@@ -11,18 +11,18 @@ sub new {
     my @V = $g->vertices;
     my $V = @V;
     my %V; @V{ @V } = 0 .. $#V;
-    bless [ [ map { [ ] } 0 .. $#V ], \%V ], $class;
+    bless [ [ map [], 0 .. $#V ], \%V ], $class;
 }
 
 sub set {
     my ($m, $u, $v, $val) = @_;
-    my ($i, $j) = map { $m->[1]->{ $_ } } ($u, $v);
+    my ($i, $j) = map $m->[1]->{ $_ }, ($u, $v);
     $m->[0]->[$i]->[$j] = $val;
 }
 
 sub get {
     my ($m, $u, $v) = @_;
-    my ($i, $j) = map { $m->[1]->{ $_ } } ($u, $v);
+    my ($i, $j) = map $m->[1]->{ $_ }, ($u, $v);
     $m->[0]->[$i]->[$j];
 }
 
@@ -35,7 +35,7 @@ sub stringify {
     for my $n (@V) {
         my $this_row = $m->[0][ $m->[1]->{$n} ];
         my @vals = map $this_row->[ $_ ], @indices;
-        push @rows, join ' ', map sprintf('%4s', $_), $n, @vals;
+        push @rows, join ' ', map sprintf('%4s', defined()?$_:''), $n, @vals;
     }
     join '', map "$_\n", $top, @rows;
 }
