@@ -1,16 +1,18 @@
 package Graph::SPTHeapElem;
 
 use strict;
-use vars qw($VERSION @ISA);
-use Heap071::Elem;
-
-use base 'Heap071::Elem';
+use warnings;
 
 sub new {
     my $class = shift;
     bless { u => $_[0], v => $_[1], w => $_[2] }, $class;
 }
 
+sub heap {
+    my $self = shift;
+    @_ ? ($self->{heap} = shift) : $self->{heap};
+}
+
 sub cmp {
     ($_[0]->{ w } || 0) <=> ($_[1]->{ w } || 0) ||
     ($_[0]->{ u } cmp $_[1]->{ u }) ||
