@@ -6,30 +6,23 @@ use warnings;
 # $SIG{__DIE__ } = \&Graph::__carp_confess;
 # $SIG{__WARN__} = \&Graph::__carp_confess;
 
-sub DEBUG () { 0 }
-
 sub reset {
     my $self = shift;
-    $self->{ unseen } = { map { $_ => $_ } $self->{ graph }->vertices };
-    $self->{ seen   } = { };
+    require Set::Object;
+    $self->{ unseen } = Set::Object->new($self->{ graph }->vertices);
+    $self->{ seen   } = Set::Object->new;
     $self->{ order     } = [ ];
     $self->{ preorder  } = [ ];
     $self->{ postorder } = [ ];
     $self->{ roots     } = [ ];
-    $self->{ tree      } =
-	Graph->new( directed => $self->{ graph }->directed );
+    $self->{ tree      } = Graph->new(directed => $self->{ graph }->directed);
     delete $self->{ terminate };
 }
 
-my $see = sub {
+sub _see {
     my $self = shift;
     $self->see;
-};
-
-my $see_active = sub {
-    my $self = shift;
-    delete @{ $self->{ active } }{ $self->see };
-};
+}
 
 sub has_a_cycle {
     my ($u, $v, $t, $s) = @_;
@@ -84,8 +77,7 @@ sub new {
 	$attr{ next_numeric } ? \&Graph::_next_numeric :
 	\&Graph::_next_random;
     $self->{ next_root } = $default_next if !exists $self->{ next_root };
-    $self->{ first_root } =
-	exists $self->{ next_root } ? $self->{ next_root } : $default_next
+    $self->{ first_root } = $self->{ next_root }
 	if !exists $self->{ first_root };
     $self->{ next_successor } = $default_next if !exists $self->{ next_successor };
     if (exists $attr{ has_a_cycle }) {
@@ -101,7 +93,7 @@ sub new {
 	$self->{ down_edge } = $find_a_cycle if $g->is_undirected;
     }
     $self->{ add } = \&add_order;
-    $self->{ see } = $see;
+    $self->{ see } = \&_see;
     delete @attr{@KNOWN_CONFIG};
     Graph::_opt_unknown(\%attr);
     return $self;
@@ -119,27 +111,17 @@ sub add_order {
 
 sub visit {
     my ($self, @next) = @_;
-    delete @{ $self->{ unseen } }{ @next };
-    print "unseen = @{[sort keys %{$self->{unseen}}]}\n" if DEBUG;
-    @{ $self->{ seen } }{ @next } = @next;
-    print "seen = @{[sort keys %{$self->{seen}}]}\n" if DEBUG;
+    $self->{ unseen }->remove(@next);
+    $self->{ seen }->insert(@next);
     $self->{ add }->( $self, @next );
-    print "order = @{$self->{order}}\n" if DEBUG;
-    if (exists $self->{ pre }) {
-	my $p = $self->{ pre };
-	for my $v (@next) {
-	    $p->( $v, $self );
-	}
-    }
+    return unless my $p = $self->{ pre };
+    $p->( $_, $self ) for @next;
 }
 
 sub visit_preorder {
     my ($self, @next) = @_;
     push @{ $self->{ preorder } }, @next;
-    for my $v (@next) {
-	$self->{ preordern }->{ $v } = $self->{ preorderi }++;
-    }
-    print "preorder = @{$self->{preorder}}\n" if DEBUG;
+    $self->{ preordern }->{ $_ } = $self->{ preorderi }++ for @next;
     $self->visit( @next );
 }
 
@@ -147,25 +129,12 @@ sub visit_postorder {
     my ($self) = @_;
     my @post = reverse $self->{ see }->( $self );
     push @{ $self->{ postorder } }, @post;
-    for my $v (@post) {
-	$self->{ postordern }->{ $v } = $self->{ postorderi }++;
-    }
-    print "postorder = @{$self->{postorder}}\n" if DEBUG;
-    if (exists $self->{ post }) {
-	my $p = $self->{ post };
-	for my $v (@post) {
-	    $p->( $v, $self ) ;
-	}
-    }
-    if (exists $self->{ post_edge }) {
-	my $p = $self->{ post_edge };
-	my $u = $self->current;
-	if (defined $u) {
-	    for my $v (@post) {
-		$p->( $u, $v, $self, $self->{ state });
-	    }
-	}
+    $self->{ postordern }->{ $_ } = $self->{ postorderi }++ for @post;
+    if (my $p = $self->{ post }) {
+	$p->( $_, $self ) for @post;
     }
+    return unless (my $p = $self->{ post_edge }) and defined(my $u = $self->current);
+    $p->( $u, $_, $self, $self->{ state }) for @post;
 }
 
 sub _callbacks {
@@ -177,39 +146,32 @@ sub _callbacks {
     my $cross    = $self->{ cross_edge };
     my $seen     = $self->{ seen_edge };
     my $bdc = defined $back || defined $down || defined $cross;
-    if (defined $nontree || $bdc || defined $seen) {
-	my $u = $current;
-	my $preu  = $self->{ preordern  }->{ $u };
-	my $postu = $self->{ postordern }->{ $u };
-	for my $v ( @all ) {
-	    my $e = $self->{ tree }->has_edge( $u, $v );
-	    if ( !$e && (defined $nontree || $bdc) ) {
-		if ( exists $self->{ seen }->{ $v }) {
-		    $nontree->( $u, $v, $self, $self->{ state })
-			if $nontree;
-		    if ($bdc) {
-			my $postv = $self->{ postordern }->{ $v };
-			if ($back &&
-			    (!defined $postv || $postv >= $postu)) {
-			    $back ->( $u, $v, $self, $self->{ state });
-			} else {
-			    my $prev = $self->{ preordern }->{ $v };
-			    if ($down && $prev > $preu) {
-				$down ->( $u, $v, $self, $self->{ state });
-			    } elsif ($cross && $prev < $preu) {
-				$cross->( $u, $v, $self, $self->{ state });
-			    }
-			}
+    return unless (defined $nontree || $bdc || defined $seen);
+    my $u = $current;
+    my $preu  = $self->{ preordern  }->{ $u };
+    my $postu = $self->{ postordern }->{ $u };
+    for my $v ( @all ) {
+	if (!$self->{tree}->has_edge($u, $v) && (defined $nontree || $bdc) &&
+		exists $self->{ seen }->{ $v }) {
+	    $nontree->( $u, $v, $self, $self->{ state }) if $nontree;
+	    if ($bdc) {
+		my $postv = $self->{ postordern }->{ $v };
+		if ($back &&
+		    (!defined $postv || $postv >= $postu)) {
+		    $back ->( $u, $v, $self, $self->{ state });
+		} else {
+		    my $prev = $self->{ preordern }->{ $v };
+		    if ($down && $prev > $preu) {
+			$down ->( $u, $v, $self, $self->{ state });
+		    } elsif ($cross && $prev < $preu) {
+			$cross->( $u, $v, $self, $self->{ state });
 		    }
 		}
 	    }
-	    if ($seen) {
-		my $c = $self->graph->get_edge_count($u, $v);
-		while ($c-- > 1) {
-		    $seen->( $u, $v, $self, $self->{ state } );
-		}
-	    }
 	}
+	next if !$seen;
+	my $c = $self->graph->get_edge_count($u, $v);
+	$seen->( $u, $v, $self, $self->{ state } ) while $c-- > 1;
     }
 }
 
@@ -219,30 +181,14 @@ sub next {
     my @next;
     while ($self->seeing) {
 	my $current = $self->current;
-	print "current = $current\n" if DEBUG;
-	@next = $self->{ graph }->successors( $current );
-	print "next.0 - @next\n" if DEBUG;
-	my %next; @next{ @next } = @next;
-	print "next.1 - @next\n" if DEBUG;
-	@next = values %next;
-	my @all = @next;
-	print "all = @all\n" if DEBUG;
-	delete @next{ grep exists $self->{seen}{$_}, keys %next };
-	@next = values %next;
-	print "next.2 - @next\n" if DEBUG;
-	if (@next) {
-	    @next = $self->{ next_successor }->( $self, \%next );
-	    print "next.3 - @next\n" if DEBUG;
-	    for my $v (@next) {
-		$self->{ tree }->add_edge( $current, $v );
-	    }
-	    if (exists $self->{ pre_edge }) {
-		my $p = $self->{ pre_edge };
-		my $u = $self->current;
-		for my $v (@next) {
-		    $p->( $u, $v, $self, $self->{ state });
-		}
-	    }
+	my $next = Set::Object->new($self->{ graph }->successors($current));
+	my @all = $next->members;
+	$next = $next->difference($self->{seen});
+	if ($next->size) {
+	    @next = $self->{ next_successor }->( $self, { map +($_=>$_), $next->members } );
+	    $self->{ tree }->add_edges(map [$current, $_], @next);
+	    last unless my $p = $self->{ pre_edge };
+	    $p->($current, $_, $self, $self->{ state }) for @next;
 	    last;
 	} else {
 	    $self->visit_postorder;
@@ -250,31 +196,18 @@ sub next {
 	return undef if $self->{ terminate };
 	$self->_callbacks($current, @all);
     }
-    print "next.4 - @next\n" if DEBUG;
     unless (@next) {
-	unless ( @{ $self->{ roots } } ) {
-	    my $first = $self->{ first_root };
-	    if (defined $first) {
-		@next =
-		    ref $first eq 'CODE' ? 
-			$self->{ first_root }->( $self, $self->{ unseen } ) :
-			$first;
-		return unless @next;
-	    }
+	if (!@{ $self->{ roots } } and defined(my $first = $self->{ first_root })) {
+	    return unless @next = ref $first eq 'CODE'
+		? $first->( $self, { map +($_=>$_), $self->unseen } )
+		: $first;
 	}
-	unless (@next) {
-	    return unless defined $self->{ next_root };
-	    return unless @next =
-		$self->{ next_root }->( $self, $self->{ unseen } );
-	}
-	return if exists $self->{ seen }->{ $next[0] }; # Sanity check.
-	print "next.5 - @next\n" if DEBUG;
+	return if !@next and !$self->{ next_root };
+	return if !@next and !(@next = $self->{ next_root }->( $self, { map +($_=>$_), $self->unseen } ));
+	return if !defined $next[0] or $self->{ seen }->contains($next[0]); # Sanity check.
 	push @{ $self->{ roots } }, $next[0];
     }
-    print "next.6 - @next\n" if DEBUG;
-    if (@next) {
-	$self->visit_preorder( @next );
-    }
+    $self->visit_preorder( @next ) if @next;
     return $next[0];
 }
 
@@ -296,12 +229,12 @@ sub postorder {
 
 sub unseen {
     my $self = shift;
-    values %{ $self->{ unseen } };
+    $self->{ unseen }->${ wantarray ? \'members' : \'size' };
 }
 
 sub seen {
     my $self = shift;
-    values %{ $self->{ seen } };
+    $self->{ seen }->${ wantarray ? \'members' : \'size' };
 }
 
 sub seeing {
@@ -472,6 +405,7 @@ Called with arguments ($v, $self).
 Called when choosing the first root (start) vertex for traversal.
 Called with arguments ($self, $unseen) where $unseen is a hash
 reference with the unseen vertices as keys.
+If not supplied, defaults to the same as C<next_root>.
 
 =item next_root
 
